 //////////////////////////////////////////////////////////////////////////////
//
// PocketFrog - The Game Library for Pocket PC Devices
// Copyright 2002  Thierry Tremblay
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Thierry Tremblay makes no representations
// about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
//////////////////////////////////////////////////////////////////////////////

#include "Ripple.h"
#include "resource.h"

using namespace Frog;



//////////////////////////////////////////////////////////////////////////////
//
// _Module : This is needed to use ATL Windows
//
//////////////////////////////////////////////////////////////////////////////

CComModule _Module;



//////////////////////////////////////////////////////////////////////////////
//
// BlitSample
//
//////////////////////////////////////////////////////////////////////////////

BlitSample::BlitSample()
{

    m_config.szWindowName     = _T("Ripple Sample");
    m_config.orientation      = ORIENTATION_WEST;
    m_config.splashScreenTime = 1000;

}



bool BlitSample::GameInit()
{

		// Initialize random seed
    srand( PocketPC::GetTickCount() );
    
    DisplayDevice* display = GetDisplay();
    

    m_background = LoadImage( display, IDR_IMAGE1 );
    
    m_numbers = LoadImage( display, IDB_NUMBERS );
    m_FPSCounter = 0;

	//ripple effect init
	int size = display->GetWidth() * display->GetHeight();
	m_pBuf1 = new short[size];
	m_pBuf2 = new short[size];
	ZeroMemory(m_pBuf1, size*sizeof(short));
	ZeroMemory(m_pBuf2, size*sizeof(short));

    return true;
}



void BlitSample::GameEnd()
{
    delete m_background;
    delete m_numbers;

	//ripple effect cleanup
	delete []m_pBuf1;
	delete []m_pBuf2;
}



void BlitSample::GameLoop()
{
    DisplayDevice* display = GetDisplay();

    display->Blit( 0, 0, m_background );
    
#if defined(FROG_HPC)
    display->Blit( 320, 0, m_background );
#endif

	//ripple effect
    RippleSpread();
	DrawRippleEffect();
    
    // Update FPS
    m_FPSTicks[ m_FPSCounter & 15 ] = PocketPC::GetTickCount();
    if (m_FPSCounter > 15)
    {
        uint32_t totalTime = m_FPSTicks[ m_FPSCounter & 15 ] - m_FPSTicks[ (m_FPSCounter+1) & 15 ];
        if (totalTime == 0) totalTime = 1;
        uint32_t fps  = 16000 / totalTime;
        uint32_t n100 = fps / 100; fps -= n100 * 100;
        uint32_t n10  = fps / 10;  fps -= n10 * 10;
        uint32_t n1   = fps;

        if (n100 > 9) { n100 = 9; n10 = 9; n1 = 9; }
        
        // Display fps
        display->Blit(  0, 0, m_numbers, &Rect( n100 * 8, 0, (n100+1)*8, 11 ) );
        display->Blit(  8, 0, m_numbers, &Rect( n10 * 8, 0, (n10+1)*8, 11 ) );
        display->Blit( 16, 0, m_numbers, &Rect( n1 * 8, 0, (n1+1)*8, 11 ) );
    }
    ++m_FPSCounter;

    // Update screen
    display->Update();

}



void BlitSample::StylusDown( Point stylus )
{
	DropStone(stylus.x, stylus.y); 
}




void BlitSample::StylusUp( Point stylus )
{
	DropStone(stylus.x, stylus.y); 
}



void BlitSample::StylusMove( Point stylus )
{
	DropStone(stylus.x, stylus.y); 
}

//ripple effect
void BlitSample::RippleSpread()
{
	int width  = GetDisplay()->GetWidth();
	int height = GetDisplay()->GetHeight(); 
	for (int i=width;  i < width * (height - 1); i++)
	{
		m_pBuf2[i] = ((m_pBuf1[i - 1] + m_pBuf1[i + 1] + m_pBuf1[i - width] + m_pBuf1[i + width]) >> 1) - m_pBuf2[i];
		m_pBuf2[i] -= m_pBuf2[i] >> 5;
		if( m_pBuf2[i] != 0 )
			width = width;
	}	
	//swap the buffer
	short *ptmp =m_pBuf1;
	m_pBuf1 = m_pBuf2;
	m_pBuf2 = ptmp;
}

void BlitSample::DropStone(int x, int y, int stonesize, int stoneweight)
{		
	int width  = GetDisplay()->GetWidth();
	int height = GetDisplay()->GetHeight();
	if ((x + stonesize) > width || (y + stonesize) > height || (x - stonesize) < 0 || (y - stonesize) < 0)
		return;
	
	for (int posx = x - stonesize; posx < x + stonesize; posx++)
		for (int posy = y - stonesize; posy < y + stonesize; posy++)
			if ((posx - x) * (posx - x) + (posy - y) * (posy - y) < stonesize * stonesize)
				m_pBuf1[width * posy + posx] = -stoneweight;
}

void BlitSample::DrawRippleEffect()
{
	int width =  GetDisplay()->GetWidth();
	int height = GetDisplay()->GetHeight();
	
	Surface::LockInfo buffer, srcbuffer;
	GetDisplay()->GetBackBuffer()->Lock( buffer );
	m_background->Lock( srcbuffer );

	int dy = buffer.pitch >> 1;
    
	int xoff, yoff;
	int k = 1;

	for (int i = 1; i < height - 1; i++)
	{
		for (int j = 0; j < width; j++, k++)
		{
			xoff = m_pBuf1[k - 1] - m_pBuf1[k + 1];
			yoff = m_pBuf1[k - width] - m_pBuf1[k + width];

			if ((i + yoff ) < 0 || (i + yoff ) > height ||	(j + xoff ) < 0 || (j + xoff ) > width)
				continue;

			buffer.pixels[i * dy + j] = srcbuffer.pixels[ (i+yoff) * dy + j + xoff ];
		}
	}
	
	GetDisplay()->GetBackBuffer()->Unlock( );
	m_background->Unlock( );
}
//ripple effect end

//////////////////////////////////////////////////////////////////////////////
//
// WinMain - Entry point
//
//////////////////////////////////////////////////////////////////////////////

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPTSTR, int )
{
    _Module.Init( 0, hInstance );
    
    BlitSample game;
    
    game.Run();
    
    return 0;
}
